﻿using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Net;
using System.Text;
using System.Text.RegularExpressions;
using System.Xml;
using Chen.Common;
using Chen.DB;
using Chen.Ext;
using Newtonsoft.Json;
namespace CnBlogsCHM
{
    class Program
    {
        static string sysImage = "ContractedBlock.gif, ExpandedBlockStart.gif,copycode.gif";

        static void Main(string[] args)
        {
            Console.WriteLine("将博客备份的xml拖到到此处，回车即可！");
            var path = Console.ReadLine().TrimEnd();
            var channel = ReadCNBlogsXml(path);

            #region 准备内容的html文件
            //准备需要的参数
            //创建html存储根目录
            Directory.CreateDirectory(".//cnblogs");
            //加载html模板
            var template = File.ReadAllText(".//temChild.html");
            //复制content文件夹到根目录下
            DirectoryUtil.CopyDir(".//content", ".//cnblogs//content");
            //定义目录表 结构
            DataTable dt = new DataTable();
            dt.Columns.Add("序号", typeof(int));
            dt.Columns.Add("标题", typeof(string));
            dt.Columns.Add("创建时间", typeof(string));
            //下载html中的图片 方便离线本地浏览
            int index = 0;
            foreach (var item in channel.items)
            {
                //这里要注意一点：CHM里图片的路径用单斜杠表示
                var urls = GetHtmlImageUrls(item.description.content);
                Directory.CreateDirectory("./cnblogs/image");
                using (var client = new WebClient())
                {
                    foreach (var url in urls)
                    {
                        var fileName = Path.GetFileName(url);
                        //首先判断该图片是常用图片
                        if (sysImage.Contains(fileName))
                        {
                            //自带图片不更名
                            if (File.Exists("./cnblogs/image//" + fileName))
                            {
                                fileName = "/image/" + fileName;
                                item.description.content = item.description.content.Replace(url, ".." + fileName);
                                continue;
                            }
                            else
                            {
                                //如果是系统自带图片，则保留命名
                                fileName = "/image/" + fileName;
                            }
                        }
                        else //若图片非系统自带，则随机命名，避免重名覆盖
                        {
                            fileName = "/image/" + (index++) + Path.GetExtension(url);
                        }
                        client.DownloadFile(url, ".//cnblogs" + fileName);
                        item.description.content = item.description.content.Replace(url, ".." + fileName);
                    }
                }
                //得到单个html正文
                //以创建时间为目录分类html 
                var time = DateTime.Parse(item.pubDate);
                var content = template
                    .Replace("{channelTitle}", item.title)//博文标题
                    .Replace("{preContent}", item.description.content)//博文内容
                    .Replace("{channelHref}", item.link)//博文地址
                    .Replace("{channelLink}",channel.link)//博客地址
                    .Replace("{channelAuthor}",item.author);//博文作者
                var dir = ".//cnblogs//" + time.ToString("yyyy年MM月");
                Directory.CreateDirectory(dir);
                //html的文件名
                var name = item.title.Replace("C#", "csharp").Replace("c#", "csharp")
                    .Replace("#", string.Empty);
                File.WriteAllText(Path.Combine(dir, name + ".html"), content, Encoding.UTF8);
                //处理完毕后 将该文件添加至目录表
                var dr = dt.NewRow();
                dr["序号"] = index;
                dr["标题"] = "<a href=\"{0}/{1}.html\"'>{1}</a>".FormatString(time.ToString("yyyy年MM月"), name);
                dr["创建时间"] = time.ToString("yyyy-MM-dd HH:mm:ss");
                dt.Rows.Add(dr);
            }
            //创建目录页
            dt.TableName = "<h3>博文目录<h3>";
            DbCommon.CreateHtml(dt, false, ".//cnblogs//博文目录.html");
            #endregion

            //编译CHM文档
            ChmHelp chm = new ChmHelp();
            chm.RootPath = ".//cnblogs";
            chm.ChmFileName =Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.Desktop ),channel.title+".chm");
            chm.DefaultPage = "博文目录.html";
            chm.Title = channel.title;
            chm.Compile();
            Console.WriteLine("生成完毕 文件位于" + chm.ChmFileName);
            Directory.Delete(".//cnblogs", true);
            Console.ReadKey();
        }

        #region 得到博客对象
        /// <summary>
        /// 读取Xml文件，得到博客对象
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        static Channel ReadCNBlogsXml(string path)
        {
            try
            {
                var xml = File.ReadAllText(path);//该xml为博客园随笔备份文件
                //var xml = File.ReadAllText(@".\cnSingle.xml");//该xml为博客园随笔备份文件
                XmlDocument doc = new XmlDocument();
                doc.LoadXml(xml);
                //获取rss节点下的内容
                var channelXml = doc.SelectSingleNode("rss").InnerXml;
                //进一步细化xml格式，内容仅为rss节点下的内容
                doc.LoadXml(channelXml);
                //判断item对象个数
                var items = doc.SelectNodes("channel/item");
                if (items.Count == 1)//仅当随笔内容有1项时才会多此一举
                {
                    //1.如果仅有一个item,序列化后的json格式为 "item": {"title": "测试","link":"",....} 这样的json格式并不能序列化成List<item>的
                    //刻意去添加一个空节点 在序列化成json的格式为 "item":[{"title": "测试"...},{}...] 这样就可以序列化成List<item>了
                    var item = doc.CreateElement("item");
                    doc.FirstChild.AppendChild(item);
                }
                //将xml序列化成json，并且去掉根节点
                var json = JsonConvert.SerializeXmlNode(doc, Newtonsoft.Json.Formatting.Indented, true);
                var channel = JsonConvert.DeserializeObject<Channel>(json);
                return channel;
            }
            catch (Exception ex)
            {
                return null;
            }
        }
        #endregion

        #region 结构定义
        public class Channel
        {
            public string title { get; set; }
            public string link { get; set; }
            public string description { get; set; }
            public string language { get; set; }
            public string lastBuildDate { get; set; }
            public string pubDate { get; set; }
            public string ttl { get; set; }
            [JsonProperty("item")]
            public List<Channel_Item> items { get; set; }
        }

        public class Channel_Item
        {
            public string title { get; set; }
            public string link { get; set; }
            public string author { get; set; }
            public string pubDate { get; set; }
            public string guid { get; set; }
            public Item_Description description { get; set; }
        }

        public class Item_Description
        {
            //默认以变量名称作为json序列化的节点，由于该节点内容不符合变量定义规范，则显示指定即可
            [JsonProperty("#cdata-section")]
            public string content { get; set; }
        }
        #endregion

        #region 提取网页文件中的图片链接
        /// <summary>
        /// 提取网页文件中的图片链接
        /// </summary>
        /// <param name="sHtmlText">html</param>
        /// <returns></returns>
        public static string[] GetHtmlImageUrls(string sHtmlText)
        {
            // 定义正则表达式用来匹配 img 标签     
            Regex regImg = new Regex(@"<img\b[^<>]*?\bsrc[\s\t\r\n]*=[\s\t\r\n]*[""']?[\s\t\r\n]*(?<imgUrl>[^\s\t\r\n""'<>]*)[^<>]*?/?[\s\t\r\n]*>", RegexOptions.IgnoreCase);
            // 搜索匹配的字符串      
            MatchCollection matches = regImg.Matches(sHtmlText);
            int i = 0;
            string[] sUrlList = new string[matches.Count];
            // 取得匹配项列表        
            foreach (Match match in matches)
                sUrlList[i++] = match.Groups["imgUrl"].Value;
            return sUrlList;
        }
        #endregion
    }
}
